@@ -1,31 +1,14 @@
-use crate::util::reencode_float;
+use crate::flavor::{flavor_from_tape, reencode_float, Ck3BinaryFlavor};
 use crate::{
     detect_encoding, tokens::TokenLookup, BodyEncoding, Ck3Date, Ck3Error, Ck3ErrorKind,
-    Extraction, FailedResolveStrategy, PdsDate
+    Extraction, FailedResolveStrategy, PdsDate,
 };
-use jomini::{BinaryTape, BinaryToken, TextWriterBuilder, TokenResolver, WriteVisitor};
+use jomini::{BinaryTape, BinaryToken, TextWriterBuilder, TokenResolver};
 use std::{
     collections::HashSet,
-    io::{Cursor, Read, Write},
+    io::{Cursor, Read},
 };
 
-struct Ck3Visitor;
-impl WriteVisitor for Ck3Visitor {
-    fn visit_f32<W>(&self, mut writer: W, data: f32) -> Result<(), jomini::Error>
-    where
-        W: Write,
-    {
-        write!(writer, "{:.6}", data).map_err(|e| e.into())
-    }
-
-    fn visit_f64<W>(&self, mut writer: W, data: f64) -> Result<(), jomini::Error>
-    where
-        W: Write,
-    {
-        write!(writer, "{:.5}", data).map_err(|e| e.into())
-    }
-}
-
 /// Convert a binary gamestate to plaintext
 ///
 /// Accepted inputs:
@@ -77,16 +60,22 @@ impl Melter {
         self
     }
 
-    fn convert(
+    fn convert_flavor<F, Q>(
         &self,
-        input: &[u8],
         writer: &mut Vec<u8>,
         unknown_tokens: &mut HashSet<u16>,
-    ) -> Result<(), Ck3Error> {
-        let tape = BinaryTape::from_ck3(input)?;
+        tape: BinaryTape,
+        flavor: F,
+        resolver: &Q,
+    ) -> Result<(), Ck3Error>
+    where
+        F: Ck3BinaryFlavor,
+        Q: TokenResolver,
+    {
         let mut wtr = TextWriterBuilder::new()
             .indent_char(b'\t')
-            .indent_factor(1).from_writer_visitor(writer, Ck3Visitor);
+            .indent_factor(1)
+            .from_writer(writer);
         let mut token_idx = 0;
         let mut known_number = false;
         let mut known_unquote = false;
@@ -96,17 +85,22 @@ impl Melter {
         let mut ai_strategies_index = 0;
         let mut metadata_index = 0;
 
-        let tokens = tape.tokens();
+        // We use this to know if we are looking at a key of `ai_strategies`
+        // which is always written out as a number and not a date
+        let mut end_indices = Vec::new();
 
+        let tokens = tape.tokens();
         while let Some(token) = tokens.get(token_idx) {
             match token {
                 BinaryToken::Object(_) => {
+                    end_indices.push(token_idx);
                     wtr.write_object_start()?;
                 }
                 BinaryToken::HiddenObject(_) => {
                     wtr.write_hidden_object_start()?;
                 }
                 BinaryToken::Array(_) => {
+                    end_indices.push(token_idx);
                     wtr.write_array_start()?;
                 }
                 BinaryToken::End(x) => {
@@ -114,6 +108,7 @@ impl Melter {
                         wtr.write_end()?;
                     }
 
+                    end_indices.pop();
                     if *x == alive_data_index {
                         alive_data_index = 0;
                     }
@@ -134,8 +129,8 @@ impl Melter {
                             // If the header line is present, we will update
                             // the metadata length in bytes which is the last
                             // 8 bytes of the header line. The header line
-                            // should be 24 in length
-                            let new_size = format!("{:08x}", data.len() - 24);
+                            // should be 24 in length (with incl terminating newline)
+                            let new_size = format!("{:08x}", data.len() - 23);
                             let ns = new_size.as_bytes();
                             data[23 - ns.len()..23].copy_from_slice(ns);
                         }
@@ -145,7 +140,11 @@ impl Melter {
                 BinaryToken::U32(x) => wtr.write_u32(*x)?,
                 BinaryToken::U64(x) => wtr.write_u64(*x)?,
                 BinaryToken::I32(x) => {
-                    if known_number || ai_strategies_index != 0 {
+                    if known_number
+                        || (end_indices
+                            .last()
+                            .map_or(false, |&x| x == ai_strategies_index))
+                    {
                         write!(wtr, "{}", x)?;
                         known_number = false;
                     } else if let Some(date) = Ck3Date::from_binary_heuristic(*x) {
@@ -155,7 +154,7 @@ impl Melter {
                     }
                 }
                 BinaryToken::Quoted(x) => {
-                    if known_unquote {
+                    if known_unquote || wtr.expecting_key() {
                         wtr.write_unquoted(x.as_bytes())?;
                     } else {
                         wtr.write_quoted(x.as_bytes())?;
@@ -164,20 +163,20 @@ impl Melter {
                 BinaryToken::Unquoted(x) => {
                     wtr.write_unquoted(x.as_bytes())?;
                 }
-                BinaryToken::F32(x) => wtr.write_f32(*x)?,
+                BinaryToken::F32(x) => write!(wtr, "{:.6}", flavor.visit_f32(*x))?,
                 BinaryToken::F64(x) if !reencode_float_token => {
-                    write!(wtr, "{}", x)?;
+                    write!(wtr, "{}", flavor.visit_f64(*x))?;
                 }
                 BinaryToken::F64(x) => {
-                    let x = reencode_float(*x);
-                    if x.fract() > 1e-7 {
+                    let x = reencode_float(flavor.visit_f64(*x));
+                    if x.fract().abs() > 1e-6 {
                         write!(wtr, "{:.5}", x)?;
                     } else {
                         write!(wtr, "{}", x)?;
                     }
                     reencode_float_token = false;
                 }
-                BinaryToken::Token(x) => match TokenLookup.resolve(*x) {
+                BinaryToken::Token(x) => match resolver.resolve(*x) {
                     Some(id)
                         if self.rewrite
                             && matches!(id, "ironman" | "ironman_manager")
@@ -208,26 +207,17 @@ impl Melter {
                             ai_strategies_index = token_idx + 1;
                         }
 
-                        if matches!(id, "settings" | "setting" | "perks")
-                            || (id == "perk" && alive_data_index != 0)
+                        if matches!(
+                            id,
+                            "settings" | "setting" | "perks" | "ethnicities" | "languages"
+                        ) || (id == "perk" && alive_data_index != 0)
                         {
                             unquote_list_index = token_idx + 1;
                         }
 
                         known_number = id == "seed" || id == "random_count";
 
-                        known_unquote = unquote_list_index != 0
-                            || matches!(
-                                id,
-                                "save_game_version"
-                                    | "portraits_version"
-                                    | "meta_date"
-                                    | "color1"
-                                    | "color2"
-                                    | "color3"
-                                    | "color4"
-                                    | "color5"
-                            );
+                        known_unquote = unquote_list_index != 0 || flavor.unquote_token(id);
 
                         reencode_float_token = matches!(
                             id,
@@ -236,8 +226,10 @@ impl Melter {
                                 | "budget_short_term"
                                 | "budget_long_term"
                                 | "budget_reserved"
+                                | "damage_last_tick"
                         );
                         reencode_float_token |= alive_data_index != 0 && id == "gold";
+                        reencode_float_token &= flavor.float_reencoding();
                         wtr.write_unquoted(id.as_bytes())?;
                     }
                     None => {
@@ -278,12 +270,34 @@ impl Melter {
             token_idx += 1;
         }
 
+        let inner = wtr.into_inner();
+        inner.push(b'\n');
         Ok(())
     }
 
-    /// Given one of the accepted inputs, this will return the save id line (if present in the input)
-    /// with the gamestate data decoded from binary to plain text.
-    pub fn melt(&self, data: &[u8]) -> Result<(Vec<u8>, HashSet<u16>), Ck3Error> {
+    fn convert<Q>(
+        &self,
+        input: &[u8],
+        resolver: &Q,
+        writer: &mut Vec<u8>,
+        unknown_tokens: &mut HashSet<u16>,
+    ) -> Result<(), Ck3Error>
+    where
+        Q: TokenResolver,
+    {
+        let tape = BinaryTape::from_slice(input)?;
+        let flavor = flavor_from_tape(&tape);
+        self.convert_flavor(writer, unknown_tokens, tape, flavor, resolver)
+    }
+
+    pub fn melt_with_tokens<Q>(
+        &self,
+        data: &[u8],
+        resolver: &Q,
+    ) -> Result<(Vec<u8>, HashSet<u16>), Ck3Error>
+    where
+        Q: TokenResolver,
+    {
         let mut result = Vec::with_capacity(data.len());
         let mut unknown_tokens = HashSet::new();
 
@@ -309,7 +323,9 @@ impl Melter {
         let mut reader = Cursor::new(data);
 
         match detect_encoding(&mut reader)? {
-            BodyEncoding::Plain => self.convert(data, &mut result, &mut unknown_tokens)?,
+            BodyEncoding::Plain => {
+                self.convert(data, resolver, &mut result, &mut unknown_tokens)?
+            }
             BodyEncoding::Zip(mut zip) => {
                 let size = zip
                     .by_name("gamestate")
@@ -327,7 +343,7 @@ impl Melter {
                         zip_file
                             .read_to_end(&mut inflated_data)
                             .map_err(|e| Ck3ErrorKind::ZipExtraction("gamestate", e))?;
-                        self.convert(&inflated_data, &mut result, &mut unknown_tokens)?
+                        self.convert(&inflated_data, resolver, &mut result, &mut unknown_tokens)?
                     }
 
                     #[cfg(feature = "mmap")]
@@ -335,7 +351,7 @@ impl Melter {
                         let mut mmap = memmap::MmapMut::map_anon(zip_file.size() as usize)?;
                         std::io::copy(&mut zip_file, &mut mmap.as_mut())
                             .map_err(|e| Ck3ErrorKind::ZipExtraction("gamestate", e))?;
-                        self.convert(&mmap[..], &mut result, &mut unknown_tokens)?
+                        self.convert(&mmap[..], resolver, &mut result, &mut unknown_tokens)?
                     }
                 }
             }
@@ -343,4 +359,10 @@ impl Melter {
 
         Ok((result, unknown_tokens))
     }
+
+    /// Given one of the accepted inputs, this will return the save id line (if present in the input)
+    /// with the gamestate data decoded from binary to plain text.
+    pub fn melt(&self, data: &[u8]) -> Result<(Vec<u8>, HashSet<u16>), Ck3Error> {
+        self.melt_with_tokens(data, &TokenLookup)
+    }
 }
